//
// BuildIndexWindow.cpp - the dialog used when in 'background' mode.
//

#include "StdHdr.h"

#include "BuildIndexWindow.h"

#include "Res\Resource.h"

#include "FastFindApp.h"

#include "BinaryFileIndexWriter.h"
#include "VolumeIndexBuilder.h"

BuildIndexWindow::BuildIndexWindow() : vgui::Dialog(IDD_BUILDINDEX),
									   m_pIndexBuilder(NULL),
									   m_pIndexFile(NULL),
									   m_pIndexDiskFile(NULL)
{
	// TCILB
}

BuildIndexWindow::~BuildIndexWindow()
{
}

void BuildIndexWindow::OnInitDialog()
{
	// Attach controls
	m_IndexStatus.Attach(this, IDC_INDEXSTATUS);

	// Set up a timer to trigger the index building, and allow
	// us to monitor the progress.
	vgui::Timer::SetTimer(vgui::TimerType_Periodic, 100, this);
	
	// Minimise the window (otherwise you will probably cancel the index
	// building accidentally, due to focus stealing)
	SetPosture(vgui::WindowPosture_Minimised);
}

bool BuildIndexWindow::OnTimer(vgui::TimerID ID)
{
	// If we haven't started the indexing, then do it now...
	if (m_pIndexBuilder == NULL)
	{
		// Create the index file...
		m_pIndexDiskFile = new vos::DiskFile;
		vos::FileSpec fsIndexFileName = FastFindApp::GetIndexFileSpec();

		if (m_pIndexDiskFile->Open(fsIndexFileName, vos::fam_Write))
		{
			// Index file created so now fill it with data.
			m_pIndexFile = new BinaryFileIndexWriter(m_pIndexDiskFile);
			m_pIndexBuilder = new VolumeIndexBuilder;
			m_pIndexBuilder->StartIndexing(m_pIndexFile);
		}
		else
		{
			// Unable to open index file - tell user.
			REPORT_ERROR();
		}
	}
	else
	{
		// just check to see if the index has finished.
		vos::FileSpec fsFile;
		if (m_pIndexBuilder->IsIndexing(&fsFile))
		{
			// Update the dialog to show the current folder being
			// indexed...
			m_IndexStatus.SetText(fsFile.GetString());
		}
		else
		{
			// We've finished - shut down the app...
			vgui::Timer::KillTimer(ID);
			OnCancel();
		}
	}

	// Message processed
	return true;
}

void BuildIndexWindow::OnCancel()
{
	// If still indexing, then stop indexing.
	if (m_pIndexBuilder != NULL)
	{
		StopIndexing();
	}

	// OK, now quit the app
	Dialog::OnCancel();
}

/**************************************************************************

>	void BuildIndexWindow::StopIndexing()

	Author:		Tim Browse
	Date:		2001/10/25

	Purpose:	Stop the ongoing index building operation.

**************************************************************************/

void BuildIndexWindow::StopIndexing()
{
	if (m_pIndexBuilder->IsIndexing())
		m_pIndexBuilder->StopIndexing();

 	// And clean up indexing objects.
	ShutdownIndexing();
}

/**************************************************************************

>	void BuildIndexWindow::ShutdownIndexing()

	Author:		Tim Browse
	Date:		28/04/2001

	Purpose:	Clean up the objects used by the dialog to perform index 
				building.  The index file is also closed if necessary.

**************************************************************************/

void BuildIndexWindow::ShutdownIndexing()
{
	// Destroy index file writer
	delete m_pIndexFile;
	m_pIndexFile = NULL;

	// Close and destroy file object used to create index file.
	if (m_pIndexDiskFile != NULL)
	{
		if (m_pIndexDiskFile->IsOpen())
			m_pIndexDiskFile->Close();
		delete m_pIndexDiskFile;
		m_pIndexDiskFile = NULL;
	}

	// And delete the indexer itself.
	delete m_pIndexBuilder;
	m_pIndexBuilder = NULL;
}

