//
// FastFindApp.cpp : The main application class.
//

#include "stdhdr.h"

// Application declaration
#include "FastFindApp.h"

// Application window(s)
#include "FastFindWindow.h"
#include "BuildIndexWindow.h"

// For resource IDs
#include "Res\Resource.h"

// For redrawing the window contents
#include "vgui\RedrawGC.h"

#include "vgui\ErrorReporter.h"
#include "vgui\Alert.h"
#include "vgui\Question.h"
#include "vgui\Errors.h"

#include "vos\Preferences.h"
#include "vos\Folders.h"

//
// Enable vOS memory tracking
//
#ifdef _DEBUG
#define new vosDEBUG_NEW
#endif

namespace vgui
{

	/**************************************************************************

	>	Application *CreateApplication()

		Purpose:	Create the instance of the application object we want to
					use for this program.

					This is called by the vGUI library on startup, and is how
					we control what type of application is created.

	**************************************************************************/

	Application *CreateApplication()
	{
		return new FastFindApp;
 	}
};

//
// Message map for the application - minimal functionality.
//
// Here we simply link the IDM_EXIT menu command to the vgui 
// Application::Exit() function, so that when the user chooses
// Exit from the menu, the program will exit.
//
BEGIN_VGUI_MESSAGE_MAP(FastFindApp, Application)
	ON_VGUI_COMMAND(IDM_EXIT, Exit)
END_VGUI_MESSAGE_MAP()


// Location of the index file
vos::FileSpec FastFindApp::s_fsIndexLocation;

// Number of results to show at once (default value)
int FastFindApp::s_cMaxResults = 1000;

/**************************************************************************

>	FastFindApp::FastFindApp()

	Author:		vGUI AppWizard

	Purpose:	Constructor - sets up the main application object.

				In this function, we tell the vGUI library what the
				application is called, which menu resource to use,
				which hotkey/accelerator resource to use, and initialise
				any optional vGUI sub-systems that we will use.

**************************************************************************/

FastFindApp::FastFindApp()
{
	// Set the app title.
	m_resAppTitle = vgui::StringResourceID(IDS_APP_TITLE);

	// Set the menu resource ID
//	m_resMenu = vgui::MenuResourceID(IDM_FASTFIND);

	// Set the main application icon resource ID
	m_resIcon = vgui::IconResourceID(IDI_FASTFIND);

	// Set keyboard accelerators
	m_resAccelerators = vgui::AcceleratorResourceID(IDC_ACCEL_FASTFIND);

	// Set up Alert system
	vgui::Alert::Initialise(IDD_VGUI_ALERT, IDC_ALERT_TEXT, IDOK);

	// Set up Question system
	vgui::Question::Initialise(IDD_VGUI_QUESTION, 
							   IDC_QUESTION_TEXT, 
							   IDC_VGUI_QUESTION_DEFAULT,
							   IDC_VGUI_QUESTION_OTHER);

	// Set up error handlers so that we get a dialog instead of the vOS
	// default of trace output and a debugger interrupt.
	SetupErrorReporter();

	// Initialise preferences
	vos::Preferences::Init("Guys with Towels", "FastFind");
	ReadPrefs();
}



/**************************************************************************

>	void FastFindApp::SetupErrorReporter()

	Author:		vGUI AppWizard

	Purpose:	Set up the vOS error system to use the standard vGUI
				error reporting dialog system.  If you want to use your
				own system, simply derive your own class from 
				vos::ErrorReporter and tell vOS to use that instead.

**************************************************************************/

void FastFindApp::SetupErrorReporter()
{
	vgui::ErrorReporterInfo Info;

	// Dialog for debug builds...
	Info.m_ErrorDialog_Debug = IDD_VGUI_REPORTERROR_DEBUG;
	Info.m_ContinueButton = IDC_VGUI_CONTINUE;
	Info.m_DebugButton = IDC_VGUI_DEBUG;
	Info.m_AbortButton = IDC_VGUI_ABORT;
	Info.m_ErrorText = IDC_VGUI_ERRORTEXT;
	Info.m_ErrorDetailsText = IDC_VGUI_ERRORDETAILS;
	Info.m_MoreInfoButton = IDC_VGUI_MOREINFO;
	Info.m_PrevErrorButton = IDC_VGUI_PREVERROR;

	// Dialog for release builds...
	Info.m_ErrorDialog_Release = IDD_VGUI_REPORTERROR_RELEASE;

	// Use this information to create a vGUI based error reporter.
	vgui::vGuiErrorReporter* pMyErrorReporter = new vgui::vGuiErrorReporter(Info);

	// And install it.
	vos::TheErrorManager().SetErrorReporter(pMyErrorReporter);

	// Initialise preference system
	vos::Preferences::Init("Guys with Towels", "FastFind");
}


/**************************************************************************

>	vgui::Dialog* FastFindApp::CreateTheDialog()

	Author:		vGUI AppWizard
	Returns:	The main Dialog object for the application.

	Purpose:	Create the main dialog window for the application

**************************************************************************/

vgui::Dialog* FastFindApp::CreateTheDialog()
{
	// Work out which dialog to create...
	vos::String CmdLine = GetCommandLine();

	if (CmdLine.Find("/buildindex"))
	{
		// Run the dialog that rebuilds the index
		return new BuildIndexWindow;
	}
	else
	{
		// Just run the normal app...
		return new FastFindWindow(this);
	}
}


/**************************************************************************

>	vos::FileSpec FastFindApp::GetIndexFileSpec()

	Author:		Tim Browse
	Date:		2001/10/25
	Returns:	FileSpec describing the name of the index file.

	Purpose:	Find out the filename of the index file.

**************************************************************************/

vos::FileSpec FastFindApp::GetIndexFileSpec()
{
	vos::FileSpec fsIndexFileName(s_fsIndexLocation);
	fsIndexFileName.AppendFilename(kIndexFile);
	return fsIndexFileName;
}

/**************************************************************************

>	vos::FileSpec FastFindApp::GetIndexLocationFileSpec()

	Author:		Tim Browse
	Date:		2001/10/25
	Returns:	FileSpec describing the location of the index file.

	Purpose:	Find out the folder where the index file should be kept.

**************************************************************************/

vos::FileSpec FastFindApp::GetIndexLocationFileSpec()
{
	return s_fsIndexLocation;
}

/**************************************************************************

>	void FastFindApp::SetIndexLocationFileSpec(const vos::FileSpec& fsNewLocation)

	Author:		Tim Browse
	Date:		1/12/2001

	Inputs:		fsNewLocation - the new location for the index file.

	Purpose:	Change the preference setting for the location of the
				index file.  This function does nothing other than change
				the filename in the preferences - if you want the index
				file to be moved, you have to do it yourself.

**************************************************************************/

void FastFindApp::SetIndexLocationFileSpec(const vos::FileSpec& fsNewLocation)
{
	s_fsIndexLocation = fsNewLocation;
	vos::GetPreferences().Write("IndexLocation", s_fsIndexLocation);
}

/**************************************************************************

>	void FastFindApp::ReadPrefs()

	Author:		Tim Browse
	Date:		1/12/2001

	Purpose:	Read preferences values from the file that need
				special fallback handling if they are missing.

				Called on app startup.

**************************************************************************/

void FastFindApp::ReadPrefs()
{
	// Read index locaton preference setting
	vos::GetPreferences().Read("IndexLocation", &s_fsIndexLocation);

	// Use location of EXE if no location is given.
	if (s_fsIndexLocation.IsEmpty())
	{
		if (vos::Folders::ApplicationHome(&s_fsIndexLocation))
		{
			// Probably the first time this was run, so write the
			// preference back to the registry now we've got it.
			vos::GetPreferences().Write("IndexLocation", s_fsIndexLocation);

		}
		else
		{
			// Can't even find out where the EXE is! Oh well...
			ERROR_HANDLED();
		}
	}

	// Read maximum results setting
	vos::GetPreferences().Read("MaxResults", &s_cMaxResults);
	ValidateMaxResults();
}

/**************************************************************************

>	void FastFindApp::ValidateMaxResults()

	Author:		Tim Browse
	Date:		30/3/2002

	Purpose:	Ensure that the static member variable s_cMaxResults is
				a valid number, i.e. a multiple of 1,000 between 1,000
				and 50,000.

**************************************************************************/

void FastFindApp::ValidateMaxResults()
{
	// Result count must be between 1,000 and 50,000, and be a multiple
	// of 1,000.

	const int kMultiple = 1000;

	s_cMaxResults = ((s_cMaxResults + kMultiple - 1) / kMultiple) * kMultiple;

	if (s_cMaxResults < (1 * kMultiple))
	{
		s_cMaxResults = 1 * kMultiple;
	}

	if (s_cMaxResults > (50 * kMultiple))
	{
		s_cMaxResults = 50 * kMultiple;
	}
}

/**************************************************************************

>	int FastFindApp::GetMaximumResultsAllowed()

	Author:		Tim Browse
	Date:		30/3/2002
	Returns:	Number of search results to show at once.

	Purpose:	Find out how many results we are allowed to show at once.

**************************************************************************/

int FastFindApp::GetMaximumResultsAllowed()
{
	ValidateMaxResults();
	return s_cMaxResults;
}

/**************************************************************************

>	void FastFindApp::SetMaximumResultsAllowed(int cResults)

	Author:		Tim Browse
	Date:		30/3/2002
	Inputs:		cResults - new maximum number of results to show at once.

	Purpose:	Change the number of results we are allowed to see at
				once.

**************************************************************************/

void FastFindApp::SetMaximumResultsAllowed(int cResults)
{
	s_cMaxResults = cResults;
	ValidateMaxResults();
	vos::GetPreferences().Write("MaxResults", s_cMaxResults);
}
