//
// FastFindWindow.cpp
//

#include "stdhdr.h"

#include "FastFindWindow.h"
#include "FastFindApp.h"

#include "vgui\RedrawGC.h"
#include "vgui\Alert.h"
#include "vgui\WaitCursor.h"
#include "vgui\Shell.h"
#include "vgui\Question.h"

#include "Res\Resource.h"

#include "vos\FileMatches.h"
#include "vos\DiskFile.h"
#include "vos\Trace.h"
#include "vos\Preferences.h"
#include "vos\Time.h"
#include "vos\StringUtil.h"

#include "TextFileIndexWriter.h"
#include "BinaryFileIndexWriter.h"

#include "VolumeIndexBuilder.h"

#include "FileIndex.h"

#include "FastFindApp.h"
#include "FastFindOptions.h"
#include "ProgressDialog.h"

//
// The message map for this dialog
//

BEGIN_VGUI_MESSAGE_MAP(FastFindWindow, Dialog)
	ON_VGUI_BN_CLICKED(IDC_STARTINDEX,	OnStartStopIndex)
	ON_VGUI_BN_CLICKED(IDC_FIND,		OnFind)
	ON_VGUI_BN_CLICKED(IDC_OPTIONS,		OnOptions)
	ON_VGUI_LBN_DOUBLECLICK(IDC_SEARCHRESULTS, OnSearchResultsDoubleClick)
	ON_VGUI_EBN_TEXTCOMMIT(IDC_SEARCHFIELD, OnFind)
END_VGUI_MESSAGE_MAP()

/**************************************************************************

>	FastFindWindow::FastFindWindow(vgui::Application *pApp)

	Author:		vGUI AppWizard
	Inputs:		pApp - the parent application object.

	Purpose:	Constructor - create the main window object.

**************************************************************************/

FastFindWindow::FastFindWindow(vgui::Application *pApp) : vgui::Dialog(IDD_FASTFIND, NULL),
														  m_pIndexBuilder(NULL),
														  m_pIndexDiskFile(NULL),
														  m_pIndexFile(NULL),
														  m_bShutdownPending(false),
														  m_pIndex(NULL)
{
	// TCILB
}



/**************************************************************************

>	FastFindWindow::~FastFindWindow()

	Author:		Tim Browse
	Date:		28/04/2001

	Purpose:	Destructor

**************************************************************************/

FastFindWindow::~FastFindWindow()
{
	// If indexing has finished and needs to be shut down, then do it.
	if (m_bShutdownPending)
	{
		ShutdownIndexing();
	}

	// Delete our memory-resident index
	delete m_pIndex;
	m_pIndex = NULL;
}

/**************************************************************************

>	void FastFindWindow::OnInitDialog()

	Author:		Tim Browse
	Date:		28/04/2001

	Purpose:	Initialise the dialog and controls.

**************************************************************************/

void FastFindWindow::OnInitDialog()
{
	// Initialise base class
	Dialog::OnInitDialog();

	// Attach controls
	m_IndexButton.Attach(this, IDC_STARTINDEX);
	m_IndexStatus.Attach(this, IDC_INDEXSTATUS);
	m_Results.Attach(this, IDC_RESULTS);
	m_SearchField.Attach(this, IDC_SEARCHFIELD, "UI.SearchPattern");
	m_ResultsList.Attach(this, IDC_SEARCHRESULTS);
	m_UseRegExp.Attach(this, IDC_USEREGEXP, "Options.UseRegExp", false);

	// Set up control text
	m_IndexButton.SetText(vgui::StringResourceID(IDS_STARTINDEX));

	m_Results.SetText("No results");

	// Enable history on the search field
	m_SearchField.EnableHistoryBehaviour();
	m_SearchField.EnableCommitNotification();

	// Helpful test patterns
#if 0
	m_SearchField.CommitToHistory("*.doc");
	m_SearchField.CommitToHistory("*.txt");
	m_SearchField.CommitToHistory("*.wav");
	m_SearchField.CommitToHistory("*.avi");
	m_SearchField.CommitToHistory("*.mp3");
#endif
	
	// Load the index
	if (LoadIndex())
	{
		// Show details of index, if loaded successfully
		UpdateIndexInfo();
	}

	// Set up the controls that change size/position as dialog is resized
	AddDynamicControl(IDC_INDEXSTATUS,		vgui::ControlResize_Stretch,	vgui::ControlResize_Move);
	AddDynamicControl(IDC_STARTINDEX,		vgui::ControlResize_Static,		vgui::ControlResize_Move);
	AddDynamicControl(IDC_SEARCHRESULTS,	vgui::ControlResize_Stretch,	vgui::ControlResize_Stretch);
	AddDynamicControl(IDC_RESULTS,			vgui::ControlResize_Stretch,	vgui::ControlResize_Static);
	AddDynamicControl(IDC_OPTIONS,			vgui::ControlResize_Move,		vgui::ControlResize_Static);
	AddDynamicControl(IDC_FIND,				vgui::ControlResize_Move,		vgui::ControlResize_Static);
	AddDynamicControl(IDC_SEARCHFIELD,		vgui::ControlResize_Stretch,	vgui::ControlResize_Static);
	AddDynamicControl(IDC_USEREGEXP,		vgui::ControlResize_Move,		vgui::ControlResize_Static);

	// Ok, restore size of dialog that we had when we last ran...
	int DialogWidth, DialogHeight;

	if (vos::GetPreferences().Read("DialogWidth",  &DialogWidth) &&
		vos::GetPreferences().Read("DialogHeight", &DialogHeight))
	{
		// Read the size ok...but we refuse to make dialog smaller than it
		// already is...
		vgui::Rect rcClient;
		GetClientArea(&rcClient);

		if (DialogWidth > rcClient.GetWidth())
			rcClient.SetWidth(DialogWidth);

		if (DialogHeight > rcClient.GetHeight())
			rcClient.SetHeight(DialogHeight);

		SetClientArea(rcClient);
	}

	// Restore the window to maximised state, if that was how the user left it.
	bool bMaximised = false;
	vos::GetPreferences().Read("Maximised", &bMaximised);
	if (bMaximised)
	{
		SetPosture(vgui::WindowPosture_Maximised);
	}
}


/**************************************************************************

>	bool FastFindWindow::LoadIndex()

	Author:		Tim Browse
	Date:		29/5/2002
	Returns:	true => index loaded from disk ok
				false => an error occurred when loading the index

	Purpose:	Load the fastfind index from disk.  If there is a problem,
				we continue to use the existing index in memory (if we have
				one).

**************************************************************************/

bool FastFindWindow::LoadIndex()
{
	// Load the index
	FileIndex* pIndex = new FileIndex;
	vos::FileSpec fsIndexFilename = FastFindApp::GetIndexFileSpec();

	if (fsIndexFilename.GetFileType() == vos::FileType_File)
	{
		if (pIndex->Load(fsIndexFilename))
		{
			// Index loaded ok
			delete m_pIndex;
			m_pIndex = pIndex;
			return true;
		}
		else
		{
			// Unable to load index - display the problem in the status field.
			const vos::ErrorInfo* pErrorInfo = vos::GetLastError();

			m_IndexStatus.SetText(pErrorInfo->GetText());
			ERROR_HANDLED();
		}
	}
	else
	{
		// Index file does not exist.
		m_IndexStatus.SetText(vgui::StringResourceID(IDS_NOINDEX));
	}

	// Index load failed
	delete pIndex;
	return false;
}

/**************************************************************************

>	bool FastFindWindow::OnStartStopIndex()

	Author:		Tim Browse
	Date:		28/04/2001
	Returns:	true (always)

	Purpose:	Handle a click event on the Start/Stop indexing button.

				The indexing is started or stopped depending on the
				current state.

**************************************************************************/

bool FastFindWindow::OnStartStopIndex()
{
	// If indexing has finished and needs to be shut down, then do it.
	if (m_bShutdownPending)
	{
		ShutdownIndexing();
	}

	// Are we currently indexing?
	if (m_pIndexBuilder == NULL)
	{
		// No, so the user is asking that we start indexing...
		StartIndexing();
	}
	else
	{
		// Indexing in progress, so stop it.
		StopIndexing();
	}

	return true;
}


/**************************************************************************

>	void FastFindWindow::StartIndexing()

	Author:		Tim Browse
	Date:		2001/10/25

	Purpose:	Start an index build.  The index is built on idle events.

**************************************************************************/

void FastFindWindow::StartIndexing()
{
	// Create the index file...
	m_pIndexDiskFile = new vos::DiskFile;
	vos::FileSpec fsIndexFileName = FastFindApp::GetIndexFileSpec();

	if (m_pIndexDiskFile->Open(fsIndexFileName, vos::fam_Write))
	{
		// Index file created so now fill it with data.
		m_pIndexFile = new BinaryFileIndexWriter(m_pIndexDiskFile);

		m_pIndexBuilder = new VolumeIndexBuilder(this);

		m_pIndexBuilder->StartIndexing(m_pIndexFile);

		// Update button/edit box to reflect our status
		m_IndexButton.SetText(vgui::StringResourceID(IDS_STOPINDEX));
		m_IndexStatus.SetText(vgui::StringResourceID(IDS_BUILDINGINDEX));
	}
	else
	{
		// Unable to open index file - tell user.
		REPORT_ERROR();
	}
}

/**************************************************************************

>	void FastFindWindow::StopIndexing()

	Author:		Tim Browse
	Date:		2001/10/25

	Purpose:	Stop an ongoing index building operation.

**************************************************************************/

void FastFindWindow::StopIndexing()
{
	m_pIndexBuilder->StopIndexing();

 	// And clean up indexing objects.
	ShutdownIndexing();

	// Update button/editbox to reflect status
	m_IndexButton.SetText(vgui::StringResourceID(IDS_STARTINDEX));
	m_IndexStatus.SetText(vgui::StringResourceID(IDS_INDEXSTOPPED));
}

/**************************************************************************

>	void FastFindWindow::ShutdownIndexing()

	Author:		Tim Browse
	Date:		28/04/2001

	Purpose:	Clean up the objects used by the dialog to perform index 
				building.  The index file is also closed if necessary.

**************************************************************************/

void FastFindWindow::ShutdownIndexing()
{
	// Destroy index file writer
	delete m_pIndexFile;
	m_pIndexFile = NULL;

	// Close and destroy file object used to create index file.
	if (m_pIndexDiskFile != NULL)
	{
		if (m_pIndexDiskFile->IsOpen())
			m_pIndexDiskFile->Close();
		delete m_pIndexDiskFile;
		m_pIndexDiskFile = NULL;
	}

	// And delete the indexer itself.
	delete m_pIndexBuilder;
	m_pIndexBuilder = NULL;

	// Finished shutdown
	m_bShutdownPending = false;

	// Try to load any new index
	LoadIndex();
}


/**************************************************************************

>	void FastFindWindow::UpdateStatus(const vos::FileSpec& CurrentFolder)

	Author:		Tim Browse
	Date:		28/04/2001
	Inputs:		CurrentFolder - the folder currently being scanned by
								the index building code.

	Purpose:	Update the text control that displays the current status
				of the index file/indexing process.

				The folder being indexed is displayed in the text control.

				See IndexBuilderMonitor.

**************************************************************************/

void FastFindWindow::UpdateStatus(const vos::FileSpec& CurrentFolder)
{
	m_IndexStatus.SetText(CurrentFolder.GetString());
}


/**************************************************************************

>	void FastFindWindow::IndexComplete()

	Author:		Tim Browse
	Date:		28/04/2001

	Purpose:	Called when the index building is complete.

				The indexing objects are cleaned up, and the text status
				control in the dialog is updated to show the status of 
				the index file.

				See IndexBuilderMonitor.

**************************************************************************/

void FastFindWindow::IndexComplete()
{
	if (LoadIndex())
	{
		UpdateIndexInfo();
	}

	m_bShutdownPending = true;
}


/**************************************************************************

>	void FastFindWindow::UpdateIndexInfo()

	Author:		Tim Browse
	Date:		28/04/2001

	Purpose:	Update the dialog controls when an index is not being built.

				The state and age of the index is queried and displayed in 
				the dialog.

**************************************************************************/

void FastFindWindow::UpdateIndexInfo()
{
	// Set button to say "Start Indexing"
	m_IndexButton.SetText(vgui::StringResourceID(IDS_STARTINDEX));

	//
	// Find out state of the index...
	//
	vgui::StringResourceID IndexStatus = IDS_INDEXSTATUS_UNKNOWN;

	vos::FileSpec fsIndexFileName = FastFindApp::GetIndexFileSpec();

	// Does the index exist?
	if (fsIndexFileName.GetFileType() == vos::FileType_File)
	{
		// Yes, query its status - it's on the first line of the file
		vos::DiskFile IndexDiskFile;
		if (IndexDiskFile.Open(fsIndexFileName, vos::fam_Read))
		{
			// Read the header
			BinaryIndexHeader Header;
			if (IndexDiskFile.Read(&Header, sizeof(Header)))
			{
				if (Header.m_Status == IndexStatus_Good)
				{
					IndexStatus = IDS_INDEXSTATUS_GOOD;
				}
				else
				{
					IndexStatus = IDS_INDEXSTATUS_BAD;
				}
			}
			else
			{
				// Unable to read index header => status unknown
				ERROR_HANDLED();
			}

			// All done with the index file
			IndexDiskFile.Close();
		}
		else
		{
			// Unable to open file for reading => status unknown
			ERROR_HANDLED();
		}
	}
	else
	{
		// Index file does not exist
		m_IndexStatus.SetText("Index not found.");
		return;
	}

	// Update the index status display, including how old the index is.
	vos::Time IndexTimeStamp;
	fsIndexFileName.GetFileTime(&IndexTimeStamp);
	UpdateIndexInfo(IndexStatus, IndexTimeStamp);
}


/**************************************************************************

>	void FastFindWindow::UpdateIndexInfo(vgui::StringResourceID IndexStatus, 
										 const vos::Time& IndexTimeStamp)

	Author:		Tim Browse
	Date:		28/04/2001
	Inputs:		IndexStatus - the string to use to display the info
				IndexTimeStamp - timestamp of the index file

	Purpose:	Update the text control that displays the age and status of
				the index.

**************************************************************************/

void FastFindWindow::UpdateIndexInfo(vgui::StringResourceID IndexStatus, 
									 const vos::Time& IndexTimeStamp)
{
	// Put a friendly description of the state of the index file
	// into the display control.
	vos::String FriendlyTime;
	GetFriendlyTime(IndexTimeStamp, &FriendlyTime);

	vgui::StringResourceID rStatus(IDS_INDEXSTATUS);
	vos::String strIndexStatus;
	
	vos::String strTotalFileSize = vos::StringUtil::GetStorageSize(m_pIndex->GetTotalFileSize());
	vos::String strNumFiles = vos::StringUtil::FormatNumber(m_pIndex->GetNumberOfFiles());
	vos::String strNumFolders = vos::StringUtil::FormatNumber(m_pIndex->GetNumberOfFolders());

	rStatus.Printf(&strIndexStatus, 
				   IndexStatus.Load().c_str(),
				   FriendlyTime.c_str(), 
				   strNumFiles.c_str(),
				   strNumFolders.c_str(),
				   strTotalFileSize.c_str());

	m_IndexStatus.SetText(strIndexStatus);
}



/**************************************************************************

>	void FastFindWindow::GetFriendlyTime(const vos::Time& TimeStamp,
										 vos::String* pFriendlyTime)

	Author:		Tim Browse
	Date:		28/04/2001
	Inputs:		TimeStamp - the timestamp to convert
	Outputs:	pFriendlyTime - the user-friendly version of the time.

	Purpose:	Convert a timestamp into a user-friendly name like
				"10 minutes ago", "Yesterday", or "Last week", etc.

				If the timestamp is in the future, the function
				returns the string "in the future".

				This is fairly basic at the moment - it could do with some
				fine tuning.

**************************************************************************/

void FastFindWindow::GetFriendlyTime(const vos::Time& TimeStamp,
									 vos::String* pFriendlyTime)
{
	// Get the current time
	vos::Time Now = vos::Time::GetCurrentTime();

	vos::TimeSpan Diff = Now - TimeStamp;

	if (Diff.GetSeconds() < 0)
	{
		// Timestamp is in the future.
		*pFriendlyTime = "in the future";
		return;
	}

	// Ok, construct time description string
	vos::String Desc;

	int nSeconds = Diff.GetSeconds();
	int nMinutes = nSeconds / 60;
	int nHours	 = nMinutes / 60;
	int nDays    = nHours   / 24;
	int nWeeks   = nDays    / 7;
	int nYears   = nDays    / 365;

	if (nYears > 0)
	{
		Desc = "over a year ago";
	}
	else if (nWeeks > 1)
	{
		Desc.Printf("%d weeks ago", nWeeks);
	}
	else if (nWeeks == 1)
	{
		Desc = "last week";
	}
	else if (nDays > 1)
	{
		Desc.Printf("%d days ago", nDays);
	}
	else if (nDays == 1)
	{
		Desc = "yesterday";
	}
	else if (nHours > 1)
	{
		Desc.Printf("%d hours ago", nHours);
	}
	else if (nHours == 1)
	{
		Desc = "about an hour ago";
	}
	else if (nMinutes > 5)
	{
		int nRoughMinutes = ((nMinutes + 2) / 5) * 5;
		Desc.Printf("about %d minutes ago", nRoughMinutes);
	}
	else
	{
		Desc = "just now";
	}

	*pFriendlyTime = Desc;
}

/**************************************************************************

>	bool FastFindWindow::OnFind()

	Author:		Tim Browse
	Date:		2001/10/25
	Returns:	true (always)

	Purpose:	Search the index for the file name/pattern the user
				has requested.

**************************************************************************/

bool FastFindWindow::OnFind()
{
	if (m_pIndex == NULL)
	{
		// No index - tell the user to create one.
		vgui::Alert::Show(IDS_INDEXNOTAVAILABLE_TITLE,
						  IDS_INDEXNOTAVAILABLE, this);
		return true;
	}

	// Get the search field contents
	vos::String strPattern;
	m_SearchField.GetText(&strPattern);

	// Commit to the history buffer
	m_SearchField.CommitToHistory(strPattern);

	// Hourglass while we do this
	vgui::WaitCursor MyWaitCursor;
	UpdateWindow();
	
	// Do a search (and time it :-)
	DWORD dwBefore = ::GetTickCount();

	// Read auto-wildcard preference
	bool bAutoWildcard = false;
	vos::GetPreferences().Read(kOption_AutoWildcard, &bAutoWildcard);

	if (!m_pIndex->Search(strPattern, m_UseRegExp, bAutoWildcard, this))
	{
		// Search failed due to regexp error - tell the user.
		vgui::Alert::Show(IDS_BADREGEXP_TITLE,
						  IDS_BADREGEXP, this);
		return true;
	}

	// Find out how many results we are allowed to show at once.
	int MaxResults = FastFindApp::GetMaximumResultsAllowed();

	// If there are less than 1000 results, then display them...
	int cResults = m_pIndex->GetNumberOfSearchResults();

	// If there are too many results to display, give the user
	// a chance to see them.
	if (cResults > MaxResults)
	{
		vgui::StringResourceID Buttons[] = 
		{
			IDS_BUTTON_YES,
			IDS_BUTTON_NO,
			0
		};

		vgui::Question IncreaseResults(IDS_INCREASERESULTS_TITLE,
									   IDS_INCREASERESULTS);

		if (IncreaseResults.Ask(Buttons) == IDS_BUTTON_YES)
		{
			// User wants to be able to see the results, so change
			// their preferences.
			FastFindApp::SetMaximumResultsAllowed(cResults);
			MaxResults = FastFindApp::GetMaximumResultsAllowed();
		}
	}

	if (cResults == 0)
	{
		m_ResultsList.ClearContents();
		vgui::StringResourceID rNoResults(IDS_NORESULTS);
		vos::String strNoResults;
		rNoResults.Printf(&strNoResults, strPattern.c_str());
		m_Results.SetText(strNoResults);
	}
	else if (cResults < MaxResults)
	{
		// Display the results
		m_ResultsList.SetContents(m_pIndex);

		// Compute the time it took 
		DWORD dwAfter = ::GetTickCount();
		float fTimeTaken = ((float) (dwAfter - dwBefore)) / 1000.0;

		vos::String strResultsFound;

		uint32 cbTotalFilesSize = m_pIndex->GetSearchTotalFileSize();

		vos::String strTotalFilesSize = vos::StringUtil::GetStorageSize(cbTotalFilesSize);

		if (cResults == 1)
		{
			vgui::StringResourceID rResultsFound(IDS_ONERESULTFOUND);
			rResultsFound.Printf(&strResultsFound, strPattern.c_str(), fTimeTaken);
		}
		else
		{
			vgui::StringResourceID rResultsFound(IDS_RESULTSFOUND);
			rResultsFound.Printf(&strResultsFound, 
								 vos::StringUtil::FormatNumber(cResults).c_str(), 
								 strPattern.c_str(), 
								 fTimeTaken, 
								 strTotalFilesSize.c_str());
		}

		m_Results.SetText(strResultsFound);
	}
	else
	{
		// Tell user there are too many results to display.
		m_ResultsList.ClearContents();
		vgui::StringResourceID rTooManyResults(IDS_TOOMANYRESULTS);
		vos::String strTooManyResults;
		rTooManyResults.Printf(&strTooManyResults, MaxResults);
		m_Results.SetText(strTooManyResults);
	}
	
	return true;
}


/**************************************************************************

>	bool FastFindWindow::OnOptions()

	Author:		Tim Browse
	Date:		2001/10/25
	Returns:	true (always)

	Purpose:	Display the program options dialog.

**************************************************************************/

bool FastFindWindow::OnOptions()
{
	FastFindOptions OptionsDialog(this);
	OptionsDialog.DoModal();

	return true;
}

/**************************************************************************

>	void FastFindWindow::OnCancel()

	Author:		Tim Browse
	Date:		2001/10/28

	Purpose:	Handle closing of the app - all we do here is remember
				the size of the dialog, and restore it next time we run.

**************************************************************************/

void FastFindWindow::OnCancel()
{
	// Remember current window size, so we can restore it next time.
	vgui::Rect rcClient;
	GetClientArea(&rcClient);

	vos::GetPreferences().Write("DialogWidth", rcClient.GetWidth());
	vos::GetPreferences().Write("DialogHeight", rcClient.GetHeight());

	// Also remember whether the dialog was maximised...
	bool bMaximised = (GetPosture() == vgui::WindowPosture_Maximised);
	vos::GetPreferences().Write("Maximised", bMaximised);

	// Make sure we store autoprefs (as this is cancel handler, it won't
	// be done automatically)
	CommitAutoPreferences();

	// Use base class to close dialog...
	Dialog::OnCancel();
}

/**************************************************************************

>	bool FastFindWindow::OnSearchResultsDoubleClick(int iItem)

	Author:		Tim Browse
	Date:		2001/10/28
	Returns:	true (message handled)

	Purpose:	Handle the user double-clicking on an item in the search
				results - we open the folder that contains the item they
				clicked on.

**************************************************************************/

bool FastFindWindow::OnSearchResultsDoubleClick(int iItem)
{
	// Find out which item is selected...
	vos::FileSpec fsFolder;
	if (m_ResultsList.GetSelectedFolder(&fsFolder))
	{
		// Ok, open this folder on the desktop for the user
		if (!vgui::Shell::OpenFolder(fsFolder))
		{
			REPORT_ERROR();
		}
	}
	else
	{
		COMPLAIN("Received double-click event but no folder was selected!");
	}

	// Message handled.
	return true;
}
