//
// FileMover.h
//

#ifndef inc_FileMover_h
#define inc_FileMover_h

#include "vos\FileSpec.h"

#include "ProgressWorker.h"

//
// TFileMoveResult
//
// Specifies the result of trying to move a file with the FileMover class.
//
typedef enum
{
	// File was moved ok
	FileMover_Success,

	//
	// Error conditions...
	//

	// User cancelled the operation partway through
	FileMover_UserCancelled,

	// Not enough space to copy the index
	FileMover_NoSpace,

	// The destination file already exists
	FileMover_FileAlreadyExists,

	// The file had to be moved using copy/delete, but the
	// copy failed.
	FileMover_CopyFailed,

	// The file had to be moved using copy/delete, but the
	// delete failed.
	FileMover_DeleteFailed
}
TFileMoveResult;

/**************************************************************************
				  
	class FileMover : public vos::CopyFileHandler,
					  public ProgressWorker_OneShot

	Author:		Tim Browse

	Date:		2001/10/27

	Purpose:	Move a file from one place to another, acting in
				conjunction with the ProgressDialog class.

				Example usage:											 */
#if 0
					// Set up the worker object
					FileMover Worker(vos::FileSpec("c:\\source.dat"), 
									 vos::FileSpec("c:\\dest.dat"));

					// Create the progress dialog, and set the operation
					// going...
					ProgressDialog Dialog(&Worker, "Moving file...");
					Dialog.DoModal();

					// Find out how the operation went
					if (Worker.GetResultCode() != FileMover_Success)
					{
						// Something went wrong...
					}

#endif
		
/*************************************************************************/

class FileMover : public vos::CopyFileHandler,
				  public ProgressWorker_OneShot
{
	public:
		// Constructor
		FileMover(const vos::FileSpec& SourceFile,
				  const vos::FileSpec& DestFile);

		// Find out how the operation went.
		TFileMoveResult GetResultCode() const { return m_ResultCode; }

		// If an error occurred, get the text of the error message
		// (if available)
		bool GetErrorText(vos::String* pErrorMsg);

	public:
		// CopyFileHandler interface
		virtual vos::Error CopyFileCallback(uint cBytesCopied);

		// ProgressWorker_OneShot interface
		virtual void Prepare();
		virtual void DoWork(ProgressDialog* pProgressDialog);

	private:
		// The progress dialog
		ProgressDialog* m_pDialog;

		// Source and destination pathnames
		vos::FileSpec m_fsSource;
		vos::FileSpec m_fsDest;

		// Size of the file we will be copying
		uint32 m_cbFileSize;

		// Result of the operation
		TFileMoveResult m_ResultCode;

		// DId the user cancel the operation?
		bool m_bCancelled;

		// Text of any error that occured.
		vos::String m_ErrorMsg;
};

#endif // !inc_FileMover_h
