//
// ProgressDialog.h
//

#ifndef inc_ProgressDialog_h
#define inc_ProgressDialog_h

#include "vgui\Dialog.h"
#include "vgui\ProgressBar.h"
#include "vgui\Button.h"
#include "vgui\Application.h"

#include "ProgressWorker.h"

/**************************************************************************

	class ProgressDialog : public vgui::Dialog

	Author:		Tim Browse

	Date:		2001/10/25

	Purpose:	Display a progress dialog while some work is being done.

				The client provides a ProgressWorker object that does
				the actual work.  The object can either do the job a
				bit at a time, and allow the progress dialog to update
				the progress bar automatically, or the worker object
				can do the job in one function call, and ask the
				progress dialog to update itself when it is convenient
				for the worker.

				The ProgressWorker object can do work in a piecemeal
				fashion, and keep returning 

**************************************************************************/

class ProgressDialog : public vgui::Dialog
{
	public:
		// Constructors - use the one depending on what type of
		// ProgressWorker object you want to use.
		ProgressDialog(ProgressWorker_Timesliced* pWorker,
					   const vos::String& strProgressMessage,
					   vgui::Window* pParent = NULL);

		ProgressDialog(ProgressWorker_OneShot* pWorker,
					   const vos::String& strProgressMessage,
					   vgui::Window* pParent = NULL);

		// Update the progress bar
		void UpdateProgressDisplay(int PercentComplete);

		// Find out if the user has cancelled the operation yet.
		bool Cancelled() const;

		// Dialog init/message handling
		void OnInitDialog();
		virtual bool OnTimer(vgui::TimerID ID);
		virtual void OnCancel();

	private:
		// Central function for changing slider position
		void ProgressDialog::SetPosition(int PercentComplete);

	private:
		// The progress bar control
		vgui::ProgressBar m_ProgressBar;

		// The cancel button
		vgui::Button m_Cancel;

		// The text to display as the progress message
		vos::String m_strProgressMessage;

		// The worker object (only one type used at any one time).
		ProgressWorker_Timesliced*  m_pTimeslicedWorker;
		ProgressWorker_OneShot*		m_pOneShotWorker;

		// Timer ID for our timer events
		vgui::TimerID m_TimerID;

		// Flag to remember whether 'Cancel' has been pressed yet.
		bool m_bCancelled;
};


#endif //! inc_ProgressDialog_h
